#ifdef _OPENMP
#include <omp.h>
#endif

#include <string>

#include "Vec3f.hxx"
#include "Image.hxx"
#include "PerspectiveCamera.hxx"
#include "Scene.hxx"


void RenderFrameCone(const std::string& fileName)
{
  /* Scene definition */
  Scene scene;
  
  scene.ParseOBJ("cone.obj", 1.0f);
 
  // alter the camera definition appropriately to see the cow
  // you may need to implement some set/get routines for the scene class
  scene.setCamera(new PerspectiveCamera(Vec3f(0,0,0.5),
					Vec3f(0,0,-1),
					Vec3f(0,1,0),
					60,
					640,
					480));
  
  
  Image img(scene.camera()->resX(),scene.camera()->resY()); // image array
  // primary ray
#pragma omp parallel for
  for(int y = 0; y < scene.camera()->resY(); y++)
    {
      for (int x = 0; x < scene.camera()->resX(); x++) 
	{
	  
	  /* Initialize your ray here */
	  Ray ray;
	  scene.camera()->InitRay(x+0.5,y+0.5,ray); // initialize ray
	
	  Vec3f col = scene.RayTrace(ray);
	  
	  img(x,y) = col; // store pixel color
	}
    }
  img.WritePPM(fileName); // write final image
}



#define RESX 640 // image x-resolution
#define RESY 480 // image y-resolution

int main(int, char**)
{
  RenderFrameCone("cone.ppm");
}
